@@ -178,8 +178,8 @@ sharedptr<SharedConnection> ConnectionPool::connect()
         else
           cnc_string += (";DATABASE=" + default_database);
 
-        std::cout << "connecting: cnc string: " << cnc_string << std::endl;
-        //std::cout << "Glom: connecting." << std::endl;
+        //std::cout << "connecting: cnc string: " << cnc_string << std::endl;
+        std::cout << "Glom: connecting." << std::endl;
 
         //*m_refGdaConnection = m_GdaClient->open_connection(m_GdaDataSourceInfo.get_name(), m_GdaDataSourceInfo.get_username(), m_GdaDataSourceInfo.get_password() );
         //m_refGdaConnection.clear(); //Make sure any previous connection is really forgotten.
@@ -218,8 +218,8 @@ sharedptr<SharedConnection> ConnectionPool::connect()
              Glib::ustring cnc_string = "HOST=" + get_host() + ";USER=" + m_user + ";PASSWORD=" + m_password;
              cnc_string += (";DATABASE=" + default_database);
 
-             std::cout << "connecting: cnc string: " << cnc_string << std::endl;
-             //std::cout << "Glom: connecting." << std::endl;
+             //std::cout << "connecting: cnc string: " << cnc_string << std::endl;
+             std::cout << "Glom: connecting." << std::endl;
 
              Glib::RefPtr<Gnome::Gda::Connection> gda_connection =  m_GdaClient->open_connection_from_string("PostgreSQL", cnc_string);
              if(gda_connection) //If we could connect without specifying the database.
@@ -343,12 +343,16 @@ bool ConnectionPool::handle_error(bool cerr_only)
         std::cerr << "Internal error: " << error_details << std::endl;
       }
 
+      //For debugging only:
+      //Gtk::Dialog* dialog = 0;
+      //dialog->run(); //Force a crash.
+
       if(!cerr_only)
       {
-	Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Internal error")), true, Gtk::MESSAGE_WARNING );
-	dialog.set_secondary_text(error_details);
-	//TODO: dialog.set_transient_for(*get_application());
-	dialog.run();
+        Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Internal error")), true, Gtk::MESSAGE_WARNING );
+        dialog.set_secondary_text(error_details);
+        //TODO: dialog.set_transient_for(*get_application());
+        dialog.run();
       }
 
       return true; //There really was an error.
