@@ -78,22 +78,24 @@ Glib::ustring Box_Data::get_find_where_clause() const
   {
     Glib::ustring strClausePart;
 
-    const Gnome::Gda::Value data = get_entered_field_data(*(*iter));
+    const Gnome::Gda::Value data = get_entered_field_data(*iter);
 
     if(!GlomConversions::value_is_empty(data))
     {
-      const Field field = (*iter)->m_field;
-
-      bool use_this_field = true;
-      if(field.get_glom_type() == Field::TYPE_BOOLEAN) //TODO: We need an intermediate state for boolean fields, so that they can be ignored in searches.
+      const sharedptr<const Field> field = (*iter)->get_full_field_details();
+      if(field)
       {
-        if(!data.get_bool())
-          use_this_field = false;
-      }
-
-      if(use_this_field)
-      {
-        strClausePart = m_strTableName + "." + field.get_name() + " " + field.sql_find_operator() + " " +  field.sql_find(data); //% is mysql wildcard for 0 or more characters.
+        bool use_this_field = true;
+        if(field->get_glom_type() == Field::TYPE_BOOLEAN) //TODO: We need an intermediate state for boolean fields, so that they can be ignored in searches.
+        {
+          if(!data.get_bool())
+            use_this_field = false;
+        }
+  
+        if(use_this_field)
+        {
+          strClausePart = m_strTableName + "." + field->get_name() + " " + field->sql_find_operator() + " " +  field->sql_find(data); //% is mysql wildcard for 0 or more characters.
+        }
       }
     }
 
@@ -125,8 +127,7 @@ Box_Data::type_map_fields Box_Data::get_record_field_values(const Gnome::Gda::Va
 {
   type_map_fields field_values;
 
-  Field fieldPrimaryKey;
-  get_field_primary_key(fieldPrimaryKey);
+  sharedptr<const Field> fieldPrimaryKey = get_field_primary_key();
 
   Document_Glom* document = get_document();
   if(document)
@@ -138,8 +139,9 @@ Box_Data::type_map_fields Box_Data::get_record_field_values(const Gnome::Gda::Va
     type_vecLayoutFields fieldsToGet;
     for(Document_Glom::type_vecFields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
     {
-      sharedptr<LayoutItem_Field> layout_item(new LayoutItem_Field);
-      layout_item->m_field = *iter;
+      sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
+      layout_item->set_full_field_details(*iter);
+
       fieldsToGet.push_back(layout_item);
     }
 
@@ -152,7 +154,7 @@ Box_Data::type_map_fields Box_Data::get_record_field_values(const Gnome::Gda::Va
       for(Document_Glom::type_vecFields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
       {
         //There should be only 1 row. Well, there could be more but we will ignore them.
-        field_values[iter->get_name()] = data_model->get_value_at(col_index, 0);
+        field_values[(*iter)->get_name()] = data_model->get_value_at(col_index, 0);
         ++col_index;
       }
     }
@@ -167,10 +169,9 @@ Box_Data::type_map_fields Box_Data::get_record_field_values(const Gnome::Gda::Va
 
 Glib::RefPtr<Gnome::Gda::DataModel> Box_Data::record_new(bool use_entered_data, const Gnome::Gda::Value& primary_key_value)
 {
-  Field fieldPrimaryKey;
-  get_field_primary_key(fieldPrimaryKey);
+  sharedptr<const Field> fieldPrimaryKey = get_field_primary_key();
 
-  const Glib::ustring primary_key_name = fieldPrimaryKey.get_name();
+  const Glib::ustring primary_key_name = fieldPrimaryKey->get_name();
 
   type_vecLayoutFields fieldsToAdd = m_FieldsShown;
 
@@ -179,11 +180,11 @@ Glib::RefPtr<Gnome::Gda::DataModel> Box_Data::record_new(bool use_entered_data,
   for(type_vecFields::const_iterator iter = m_TableFields.begin(); iter != m_TableFields.end(); ++iter)
   {
     //TODO: Search for the non-related field with the name, not just the field with the name:
-    type_vecLayoutFields::const_iterator iterFind = std::find_if(fieldsToAdd.begin(), fieldsToAdd.end(), predicate_FieldHasName<LayoutItem_Field>(iter->get_name()));
+    type_vecLayoutFields::const_iterator iterFind = std::find_if(fieldsToAdd.begin(), fieldsToAdd.end(), predicate_FieldHasName<LayoutItem_Field>((*iter)->get_name()));
     if(iterFind == fieldsToAdd.end())
     {
-      sharedptr<LayoutItem_Field> layout_item(new LayoutItem_Field);
-      layout_item->m_field = *iter;
+      sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
+      layout_item->set_full_field_details(*iter);
 
       fieldsToAdd.push_back(layout_item);
     }
@@ -195,32 +196,34 @@ Glib::RefPtr<Gnome::Gda::DataModel> Box_Data::record_new(bool use_entered_data,
     sharedptr<LayoutItem_Field> layout_item = *iter;
 
     //If the user did not enter something in this field:
-    Gnome::Gda::Value value = get_entered_field_data(*layout_item);
+    Gnome::Gda::Value value = get_entered_field_data(layout_item);
 
     if(GlomConversions::value_is_empty(value)) //This deals with empty strings too.
     {
-      const Field& field = layout_item->m_field;
-
-      //If the default value should be calculated, then calculate it:
-      if(field.get_has_calculation())
+      const sharedptr<const Field>& field = layout_item->get_full_field_details();
+      if(field)
       {
-        const Glib::ustring calculation = field.get_calculation();
-        const type_map_fields field_values = get_record_field_values(primary_key_value);
-
-        const Gnome::Gda::Value value = glom_evaluate_python_function_implementation(field.get_glom_type(), calculation, field_values,
-          get_document(), m_strTableName);
-        set_entered_field_data(*layout_item, value);
-      }
-
-      //Use default values (These are also specified in postgres as part of the field definition,
-      //so we could theoretically just not specify it here.
-      //TODO_Performance: Add has_default_value()?
-      if(GlomConversions::value_is_empty(value))
-      {
-        const Gnome::Gda::Value default_value = field.get_default_value();
-        if(!GlomConversions::value_is_empty(default_value))
+        //If the default value should be calculated, then calculate it:
+        if(field->get_has_calculation())
         {
-          set_entered_field_data(*layout_item, default_value);
+          const Glib::ustring calculation = field->get_calculation();
+          const type_map_fields field_values = get_record_field_values(primary_key_value);
+  
+          const Gnome::Gda::Value value = glom_evaluate_python_function_implementation(field->get_glom_type(), calculation, field_values,
+            get_document(), m_strTableName);
+          set_entered_field_data(layout_item, value);
+        }
+  
+        //Use default values (These are also specified in postgres as part of the field definition,
+        //so we could theoretically just not specify it here.
+        //TODO_Performance: Add has_default_value()?
+        if(GlomConversions::value_is_empty(value))
+        {
+          const Gnome::Gda::Value default_value = field->get_default_value();
+          if(!GlomConversions::value_is_empty(default_value))
+          {
+            set_entered_field_data(layout_item, default_value);
+          }
         }
       }
     }
@@ -245,33 +248,35 @@ Glib::RefPtr<Gnome::Gda::DataModel> Box_Data::record_new(bool use_entered_data,
       {
         Gnome::Gda::Value value;
 
-        const Field& field = layout_item->m_field;
-
-        //Use the specified (generated) primary key value, if there is one:
-        if(primary_key_name == field_name && !GlomConversions::value_is_empty(primary_key_value))
+        const sharedptr<const Field>& field = layout_item->get_full_field_details();
+        if(field)
         {
-          value = primary_key_value;
-        }
-        else
-        {
-          if(use_entered_data || !field.get_calculation().empty()) //TODO_Performance: Use a get_has_calculation() method.
-            value = get_entered_field_data(*layout_item);
-        }
-
-        Glib::ustring strFieldValue = field.sql(value);
-
-        if(!strFieldValue.empty())
-        {
-          if(!strNames.empty())
+          //Use the specified (generated) primary key value, if there is one:
+          if(primary_key_name == field_name && !GlomConversions::value_is_empty(primary_key_value))
           {
-            strNames += ", ";
-            strValues += ", ";
+            value = primary_key_value;
+          }
+          else
+          {
+            if(use_entered_data || !field->get_calculation().empty()) //TODO_Performance: Use a get_has_calculation() method.
+              value = get_entered_field_data(layout_item);
+          }
+  
+          Glib::ustring strFieldValue = field->sql(value);
+  
+          if(!strFieldValue.empty())
+          {
+            if(!strNames.empty())
+            {
+              strNames += ", ";
+              strValues += ", ";
+            }
+  
+            strNames += "\"" + field_name + "\"";
+            strValues += strFieldValue;
+  
+            map_added[field_name] = true;
           }
-
-          strNames += field_name;
-          strValues += strFieldValue;
-
-          map_added[field_name] = true;
         }
       }
     }
@@ -280,7 +285,7 @@ Glib::RefPtr<Gnome::Gda::DataModel> Box_Data::record_new(bool use_entered_data,
   //Put it all together to create the record with these field values:
   if(!strNames.empty() && !strValues.empty())
   {
-    Glib::ustring strQuery = "INSERT INTO " + m_strTableName + " (" + strNames + ") VALUES (" + strValues + ")";
+    Glib::ustring strQuery = "INSERT INTO \"" + m_strTableName + "\" (" + strNames + ") VALUES (" + strValues + ")";
     return Query_execute(strQuery);
   }
   else
@@ -380,7 +385,7 @@ Gnome::Gda::Value Box_Data::generate_next_auto_increment(const Glib::ustring& ta
   //This is a workaround for postgres problems. Ideally, we need to use the postgres serial type and find out how to get the generated value after we add a row.
 
   double result = 0;
-  Glib::RefPtr<Gnome::Gda::DataModel> data_model = Query_execute("SELECT max(" + field_name + ") FROM " + table_name);
+  Glib::RefPtr<Gnome::Gda::DataModel> data_model = Query_execute("SELECT max(" + field_name + ") FROM \"" + table_name + "\"");
   if(!data_model && data_model->get_n_rows())
     handle_error();
   else
@@ -419,12 +424,11 @@ Box_Data::type_vecLayoutFields Box_Data::get_related_fields(const Glib::ustring&
       if(layout_field->get_has_relationship_name())
       {
         //Get the relationship information:
-        Relationship relationship;
-        bool test = document->get_relationship(m_strTableName, layout_field->get_relationship_name(), relationship);
-        if(test)
+        sharedptr<const Relationship> relationship = document->get_relationship(m_strTableName, layout_field->get_relationship_name());
+        if(relationship)
         {
           //If the relationship uses the specified field:
-          if(relationship.get_from_field() == field_name)
+          if(relationship->get_from_field() == field_name)
           {
             //Add it:
             result.push_back(layout_field);
@@ -447,17 +451,17 @@ Box_Data::type_field_calcs Box_Data::get_calculated_fields(const Glib::ustring&
     //Examine all fields, not just the the shown fields.
     for(type_vecFields::const_iterator iter = m_TableFields.begin(); iter != m_TableFields.end();  ++iter)
     {
-      const Field& field = *iter;
+      sharedptr<const Field> field = *iter;
 
       //Does this field's calculation use the field?
-      const Field::type_list_strings fields_triggered = field.get_calculation_fields();
+      const Field::type_list_strings fields_triggered = field->get_calculation_fields();
       Field::type_list_strings::const_iterator iterFind = std::find(fields_triggered.begin(), fields_triggered.end(), field_name);
       if(iterFind != fields_triggered.end())
       {
         CalcInProgress item;
         item.m_field = field;
 
-        result[field.get_name()] = item;
+        result[field->get_name()] = item;
       }
     }
   }
@@ -478,22 +482,21 @@ Box_Data::type_list_lookups Box_Data::get_lookup_fields(const Glib::ustring& fie
     //Examine all fields, not just the the shown fields (m_Fields):
     for(type_vecFields::const_iterator iter = m_TableFields.begin(); iter != m_TableFields.end();  ++iter)
     {
-      const Field& field = *iter;
+      sharedptr<Field> field = *iter;
 
       //Examine each field that looks up its data from a relationship:
-      if(field.get_is_lookup())
+      if(field->get_is_lookup())
       {
         //Get the relationship information:
-        Relationship relationship;
-        bool test = document->get_relationship(m_strTableName, field.get_lookup_relationship(), relationship);
-        if(test)
+        sharedptr<Relationship> relationship = field->get_lookup_relationship();
+        if(relationship)
         {
           //If the relationship is triggererd by the specified field:
-          if(relationship.get_from_field() == field_name)
+          if(relationship->get_from_field() == field_name)
           {
             //Add it:
-            LayoutItem_Field item;
-            item.m_field = field;
+            sharedptr<LayoutItem_Field> item = sharedptr<LayoutItem_Field>::create();
+            item->set_full_field_details(field);
             result.push_back( type_pairFieldTrigger(item, relationship) );
           }
         }
@@ -504,16 +507,18 @@ Box_Data::type_list_lookups Box_Data::get_lookup_fields(const Glib::ustring& fie
   return result;
 }
 
-Gnome::Gda::Value Box_Data::get_lookup_value(const Relationship& relationship, const Field& source_field, const Gnome::Gda::Value& key_value)
+Gnome::Gda::Value Box_Data::get_lookup_value(const sharedptr<const Relationship>& relationship, const sharedptr<const Field>& source_field, const Gnome::Gda::Value& key_value)
 {
   Gnome::Gda::Value result;
 
-  Field to_key_field;
-  bool test = get_fields_for_table_one_field(relationship.get_to_table(), relationship.get_to_field(), to_key_field);
-  if(test)
+  sharedptr<Field> to_key_field = get_fields_for_table_one_field(relationship->get_to_table(), relationship->get_to_field());
+  if(to_key_field)
   {
-    Glib::ustring strQuery = "SELECT " + relationship.get_to_table() + "." + source_field.get_name() + " FROM " +  relationship.get_to_table();
-    strQuery += " WHERE " + relationship.get_to_field() + " = " + to_key_field.sql(key_value);
+    //Convert the value, in case the from and to fields have different types:
+    const Gnome::Gda::Value value_to_key_field = GlomConversions::convert_value(key_value, to_key_field->get_glom_type());
+
+    Glib::ustring strQuery = "SELECT \"" + relationship->get_to_table() + "\".\"" + source_field->get_name() + "\" FROM \"" +  relationship->get_to_table() + "\"";
+    strQuery += " WHERE \"" + to_key_field->get_name() + "\" = " + to_key_field->sql(value_to_key_field);
 
     Glib::RefPtr<Gnome::Gda::DataModel> data_model = Query_execute(strQuery);
     if(data_model && data_model->get_n_rows())
@@ -530,9 +535,9 @@ Gnome::Gda::Value Box_Data::get_lookup_value(const Relationship& relationship, c
   return result;
 }
 
-void Box_Data::calculate_field(const Field& field, const Field& primary_key, const Gnome::Gda::Value& primary_key_value)
+void Box_Data::calculate_field(const sharedptr<const Field>& field, const sharedptr<const Field>& primary_key, const Gnome::Gda::Value& primary_key_value)
 {
-  const Glib::ustring field_name = field.get_name();
+  const Glib::ustring field_name = field->get_name();
   //g_warning("Box_Data::calculate_field(): field_name=%s", field_name.c_str());
 
   //Do we already have this in our list?
@@ -553,7 +558,7 @@ void Box_Data::calculate_field(const Field& field, const Field& primary_key, con
   if(refCalcProgress.m_calc_in_progress)
   {
     //g_warning("  Box_Data::calculate_field(): Circular calculation detected. field_name=%s", field_name.c_str());
-    //refCalcProgress.m_value = GlomConversions::get_empty_value(field.get_glom_type()); //Give up.
+    //refCalcProgress.m_value = GlomConversions::get_empty_value(field->get_glom_type()); //Give up.
   }
   else if(refCalcProgress.m_calc_finished)
   {
@@ -567,26 +572,24 @@ void Box_Data::calculate_field(const Field& field, const Field& primary_key, con
 
     refCalcProgress.m_calc_in_progress = true; //Let the recursive calls to calculate_field() check this.
 
-    LayoutItem_Field layout_item;
-    layout_item.m_field = refCalcProgress.m_field;
+    sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
+    layout_item->set_full_field_details(refCalcProgress.m_field);
 
     //Calculate dependencies first:
-    const Field::type_list_strings fields_needed = field.get_calculation_fields();
+    const Field::type_list_strings fields_needed = field->get_calculation_fields();
     for(Field::type_list_strings::const_iterator iterNeeded = fields_needed.begin(); iterNeeded != fields_needed.end(); ++iterNeeded)
     {
-      Field field_needed;
-      //g_warning("field %s needs %s", field_name.c_str(), iterNeeded->c_str());
-      bool test = get_document()->get_field(m_strTableName, *iterNeeded, field_needed);
-      if(test)
+      sharedptr<const Field> field_needed = get_document()->get_field(m_strTableName, *iterNeeded);
+      if(field_needed)
       {
-        if(field_needed.get_has_calculation())
+        if(field_needed->get_has_calculation())
         {
           //g_warning("  calling calculate_field() for %s", iterNeeded->c_str());
           calculate_field(field_needed, primary_key, primary_key_value);
         }
         else
         {
-          //g_warning("  not a calculated field.");
+          //g_warning("  not a calculated field->");
         }
       }
     }
@@ -604,15 +607,18 @@ void Box_Data::calculate_field(const Field& field, const Field& primary_key, con
     else
     {
       //recalculate:
+      //TODO_Performance: We don't know what fields the python calculation will use, so we give it all of them:
       const type_map_fields field_values = get_record_field_values(primary_key_value);
-      refCalcProgress.m_value  = glom_evaluate_python_function_implementation(refCalcProgress.m_field.get_glom_type(), refCalcProgress.m_field.get_calculation(), field_values,
+
+      sharedptr<const Field> field = refCalcProgress.m_field;
+      refCalcProgress.m_value  = glom_evaluate_python_function_implementation(field->get_glom_type(), field->get_calculation(), field_values,
             get_document(), m_strTableName);
 
       refCalcProgress.m_calc_finished = true;
       refCalcProgress.m_calc_in_progress = false;
 
-      LayoutItem_Field layout_item;
-      layout_item.m_field = refCalcProgress.m_field;
+      sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
+      layout_item->set_full_field_details(field);
 
       //show it:
       set_entered_field_data(layout_item, refCalcProgress.m_value );
@@ -627,9 +633,9 @@ void Box_Data::calculate_field(const Field& field, const Field& primary_key, con
 }
 
 
-void Box_Data::do_calculations(const LayoutItem_Field& field_changed, const Field& primary_key, const Gnome::Gda::Value& primary_key_value, bool first_calc_field)
+void Box_Data::do_calculations(const sharedptr<const LayoutItem_Field>& field_changed, const sharedptr<const Field>& primary_key, const Gnome::Gda::Value& primary_key_value, bool first_calc_field)
 {
-  //g_warning("Box_Data::do_calculations(): triggered by =%s", field_changed.get_name().c_str());
+  //g_warning("Box_Data::do_calculations(): triggered by =%s", field_changed->get_name().c_str());
 
   if(first_calc_field)
   {
@@ -638,16 +644,16 @@ void Box_Data::do_calculations(const LayoutItem_Field& field_changed, const Fiel
   }
 
   //Recalculate fields that are triggered by a change of this field's value, not including calculations that these calculations use.
-  type_field_calcs calculated_fields = get_calculated_fields(field_changed.get_name()); //TODO: Just return the Fields, not the CalcInProgress.
+  type_field_calcs calculated_fields = get_calculated_fields(field_changed->get_name()); //TODO: Just return the Fields, not the CalcInProgress.
   for(type_field_calcs::iterator iter = calculated_fields.begin(); iter != calculated_fields.end(); ++iter)
   {
-    const Field& field = iter->second.m_field;
+    sharedptr<const Field> field = iter->second.m_field;
 
     calculate_field(field, primary_key, primary_key_value); //And any dependencies.
 
     //Calculate anything that depends on this.
-    LayoutItem_Field layout_item;
-    layout_item.m_field = field;
+    sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
+    layout_item->set_full_field_details(field);
 
     do_calculations(layout_item, primary_key, primary_key_value, false /* recurse, reusing m_FieldsCalculationInProgress */);
   }
@@ -656,25 +662,29 @@ void Box_Data::do_calculations(const LayoutItem_Field& field_changed, const Fiel
     m_FieldsCalculationInProgress.clear();
 }
 
-bool Box_Data::set_field_value_in_database(const LayoutItem_Field& field_layout, const Gnome::Gda::Value& field_value, const Field& primary_key_field, const Gnome::Gda::Value& primary_key_value, bool use_current_calculations)
+bool Box_Data::set_field_value_in_database(const sharedptr<const LayoutItem_Field>& field_layout, const Gnome::Gda::Value& field_value, const sharedptr<const Field>& primary_key_field, const Gnome::Gda::Value& primary_key_value, bool use_current_calculations)
 {
   return set_field_value_in_database(Gtk::TreeModel::iterator(), field_layout, field_value, primary_key_field, primary_key_value, use_current_calculations);
 }
 
-bool Box_Data::set_field_value_in_database(const Gtk::TreeModel::iterator& row, const LayoutItem_Field& field_layout, const Gnome::Gda::Value& field_value, const Field& primary_key_field, const Gnome::Gda::Value& primary_key_value, bool use_current_calculations)
+bool Box_Data::set_field_value_in_database(const Gtk::TreeModel::iterator& row, const sharedptr<const LayoutItem_Field>& field_layout, const Gnome::Gda::Value& field_value, const sharedptr<const Field>& primary_key_field, const Gnome::Gda::Value& primary_key_value, bool use_current_calculations)
 {
   //row is invalid, and ignored, for Box_Data_Details.
 
-  const Field& field = field_layout.m_field;
+  const sharedptr<const Field> field = field_layout->get_full_field_details();
+  if(!field)
+    return false;
 
-  const Glib::ustring field_name = field_layout.get_name();
+  const Glib::ustring field_name = field_layout->get_name();
   if(!field_name.empty()) //This should not happen.
   {
     Glib::ustring table_name;
-    if(field_layout.get_has_relationship_name())
+    if(field_layout->get_has_relationship_name())
     {
       //The field is in a related table.
-      table_name = field_layout.m_relationship.get_to_table();
+      sharedptr<Relationship> rel = field_layout->get_relationship();
+      if(rel)
+        table_name = rel->get_to_table();
     }
     else
       table_name = m_strTableName;
@@ -685,9 +695,9 @@ bool Box_Data::set_field_value_in_database(const Gtk::TreeModel::iterator& row,
       return false;
     }
 
-    Glib::ustring strQuery = "UPDATE " + table_name;
-    strQuery += " SET " + field_name + " = " + field.sql(field_value);
-    strQuery += " WHERE " + primary_key_field.get_name() + " = " + primary_key_field.sql(primary_key_value);
+    Glib::ustring strQuery = "UPDATE \"" + table_name + "\"";
+    strQuery += " SET \"" + field_name + "\" = " + field->sql(field_value);
+    strQuery += " WHERE \"" + primary_key_field->get_name() + "\" = " + primary_key_field->sql(primary_key_value);
     Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute(strQuery);  //TODO: Handle errors
     if(!datamodel)
     {
@@ -739,15 +749,18 @@ Document_Glom::type_mapLayoutGroupSequence Box_Data::get_data_layout_groups(cons
   return layout_groups;
 }
 
-void Box_Data::fill_layout_group_field_info(LayoutGroup& group, const Privileges& table_privs)
+void Box_Data::fill_layout_group_field_info(const sharedptr<LayoutGroup>& group, const Privileges& table_privs)
 { 
+  if(!group)
+   return;
+
   const Document_Glom* document = get_document();
 
-  LayoutGroup::type_map_items items = group.get_items();
+  LayoutGroup::type_map_items items = group->get_items();
   for(LayoutGroup::type_map_items::iterator iter = items.begin(); iter != items.end(); ++iter)
   {
-    LayoutItem* item = iter->second;
-    LayoutItem_Field* item_field = dynamic_cast<LayoutItem_Field*>(item);
+    sharedptr<LayoutItem> item = iter->second;
+    sharedptr<LayoutItem_Field> item_field = sharedptr<LayoutItem_Field>::cast_dynamic(item);
     if(item_field) //If is a field rather than some other layout item
     {
 
@@ -755,19 +768,16 @@ void Box_Data::fill_layout_group_field_info(LayoutGroup& group, const Privileges
       {
         //Get the full field information:
         const Glib::ustring relationship_name = item_field->get_relationship_name();
-        Relationship relationship;
-        bool test = document->get_relationship(m_strTableName, relationship_name, relationship);
-        if(test)
+        sharedptr<const Relationship> relationship = document->get_relationship(m_strTableName, relationship_name);
+        if(relationship)
         {
-          Field field;
-          //TODO: get_fields_for_table_one_field() is probably very inefficient
-          bool test = get_fields_for_table_one_field(relationship.get_to_table(), item->get_name(), field);
-          if(test)
+          sharedptr<Field> field = get_fields_for_table_one_field(relationship->get_to_table(), item->get_name());
+          if(field)
           {
-            item_field->m_field = field;
+            item_field->set_full_field_details(field);
 
             //TODO_Performance: Don't do this repeatedly for the same table.
-            const Privileges privs = get_current_privs(relationship.get_to_table());
+            const Privileges privs = get_current_privs(relationship->get_to_table());
             item_field->m_priv_view = privs.m_view;
             item_field->m_priv_edit = privs.m_edit;
           }
@@ -776,11 +786,10 @@ void Box_Data::fill_layout_group_field_info(LayoutGroup& group, const Privileges
       else
       {
         //Get the field info:
-        Field field;
-        bool found = get_fields_for_table_one_field(m_strTableName, item_field->get_name(), field);
-        if(found)
+        sharedptr<Field> field = get_fields_for_table_one_field(m_strTableName, item_field->get_name());
+        if(field)
         {
-          item_field->m_field = field; //TODO_Performance: Just use this as the output arg?
+          item_field->set_full_field_details(field); //TODO_Performance: Just use this as the output arg?
           item_field->m_priv_view = table_privs.m_view;
           item_field->m_priv_edit = table_privs.m_edit;
         }
@@ -788,11 +797,11 @@ void Box_Data::fill_layout_group_field_info(LayoutGroup& group, const Privileges
     }
     else
     {
-      LayoutGroup* item_group = dynamic_cast<LayoutGroup*>(item);
+      sharedptr<LayoutGroup> item_group = sharedptr<LayoutGroup>::cast_dynamic(item);
       if(item_group) //If it is a group
       {
         //recurse, to fill the fields info in this group:
-        fill_layout_group_field_info(*item_group, table_privs);
+        fill_layout_group_field_info(item_group, table_privs);
       }
     }
   }
@@ -801,11 +810,10 @@ void Box_Data::fill_layout_group_field_info(LayoutGroup& group, const Privileges
 bool Box_Data::record_delete(const Gnome::Gda::Value& primary_key_value)
 {
 
-  Field field_primary_key;
-  bool test = get_field_primary_key(field_primary_key);
-  if(test && !GlomConversions::value_is_empty(primary_key_value))
+  sharedptr<Field> field_primary_key = get_field_primary_key();
+  if(field_primary_key && !GlomConversions::value_is_empty(primary_key_value))
   {
-    return Query_execute( "DELETE FROM " + m_strTableName + " WHERE " + m_strTableName + "." + field_primary_key.get_name() + " = " + field_primary_key.sql(primary_key_value) );
+    return Query_execute( "DELETE FROM \"" + m_strTableName + "\" WHERE \"" + m_strTableName + "\".\"" + field_primary_key->get_name() + "\" = " + field_primary_key->sql(primary_key_value) );
   }
   else
   {
@@ -830,11 +838,11 @@ g_warning("debug: Box_Data::get_field()");
 }
 */
 
-Glib::ustring Box_Data::build_sql_select(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const Field& primary_key_field, const Gnome::Gda::Value& primary_key_value)
+Glib::ustring Box_Data::build_sql_select(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const sharedptr<const Field>& primary_key_field, const Gnome::Gda::Value& primary_key_value)
 {
   if(!GlomConversions::value_is_empty(primary_key_value)) //If there is a record to show:
   {
-    const Glib::ustring where_clause = table_name + "." + primary_key_field.get_name() + " = " + primary_key_field.sql(primary_key_value);
+    const Glib::ustring where_clause = "\"" + table_name + "\".\"" + primary_key_field->get_name() + "\" = " + primary_key_field->sql(primary_key_value);
     return GlomUtils::build_sql_select_with_where_clause(table_name, fieldsToGet, where_clause);
   }
 
@@ -870,7 +878,7 @@ Glib::ustring Box_Data::build_sql_select_with_where_clause(const Glib::ustring&
       bool test = document->get_relationship(table_name, relationship_name, relationship);
       if(test)
       {
-        const Glib::ustring field_table_name = relationship.get_to_table();
+        const Glib::ustring field_table_name = relationship->get_to_table();
         sql_part_fields += ( field_table_name + "." );
 
         //Add the relationship to the list:
@@ -884,16 +892,16 @@ Glib::ustring Box_Data::build_sql_select_with_where_clause(const Glib::ustring&
   }
 
   result =  "SELECT " + sql_part_fields +
-    " FROM " + table_name;
+    " FROM \"" + table_name + "\"";
 
   //LEFT OUTER JOIN will get the field values from the other tables, and give us our fields for this table even if there is no corresponding value in the other table.
   Glib::ustring sql_part_leftouterjoin; 
   for(type_list_relationships::const_iterator iter = list_relationships.begin(); iter != list_relationships.end(); ++iter)
   {
     const Relationship& relationship = *iter;
-    sql_part_leftouterjoin += " LEFT OUTER JOIN " + relationship.get_to_table() +
-      " ON (" + relationship.get_from_table() + "." + relationship.get_from_field() + " = " +
-      relationship.get_to_table() + "." + relationship.get_to_field() +
+    sql_part_leftouterjoin += " LEFT OUTER JOIN " + relationship->get_to_table() +
+      " ON (" + relationship->get_from_table() + "." + relationship->get_from_field() + " = " +
+      relationship->get_to_table() + "." + relationship->get_to_field() +
       ")";
   }
 
@@ -906,16 +914,16 @@ Glib::ustring Box_Data::build_sql_select_with_where_clause(const Glib::ustring&
 }
 */
 
-bool Box_Data::get_related_record_exists(const Relationship& relationship, const Field& key_field, const Gnome::Gda::Value& key_value)
+bool Box_Data::get_related_record_exists(const sharedptr<const Relationship>& relationship, const sharedptr<const Field>& key_field, const Gnome::Gda::Value& key_value)
 {
   bool result = false;
 
   //TODO_Performance: It's very possible that this is slow.
   //We don't care how many records there are, only whether there are more than zero.
-  const Glib::ustring to_field = relationship.get_to_table() = relationship.get_to_field();
+  const Glib::ustring to_field = relationship->get_to_table() = relationship->get_to_field();
 
   //TODO_Performance: Is this the best way to just find out whether there is one record that meets this criteria?
-  const Glib::ustring query = "SELECT " + to_field + " FROM " + relationship.get_to_table() + " WHERE " + to_field + " = " + key_field.sql(key_value);
+  const Glib::ustring query = "SELECT \"" + to_field + "\" FROM \"" + relationship->get_to_table() + "\" WHERE \"" + to_field + "\" = " + key_field->sql(key_value);
   Glib::RefPtr<Gnome::Gda::DataModel> records = Query_execute(query);
   if(!records)
     handle_error();
@@ -929,7 +937,7 @@ bool Box_Data::get_related_record_exists(const Relationship& relationship, const
   return result;
 }
 
-bool Box_Data::add_related_record_for_field(const LayoutItem_Field& layout_item_parent, const Relationship& relationship, const Field& primary_key_field, const Gnome::Gda::Value& primary_key_value_provided)
+bool Box_Data::add_related_record_for_field(const sharedptr<const LayoutItem_Field>& layout_item_parent, const sharedptr<const Relationship>& relationship, const sharedptr<const Field>& primary_key_field, const Gnome::Gda::Value& primary_key_value_provided)
 {
   Gnome::Gda::Value primary_key_value = primary_key_value_provided;
 
@@ -941,7 +949,7 @@ bool Box_Data::add_related_record_for_field(const LayoutItem_Field& layout_item_
   else
   {
     //To store the entered data in the related field, we would first have to create a related record.
-    if(!relationship.get_auto_create())
+    if(!relationship->get_auto_create())
     {
       //Warn the user:
       //TODO: Make the field insensitive until it can receive data, so people never see this dialog.
@@ -957,7 +965,7 @@ bool Box_Data::add_related_record_for_field(const LayoutItem_Field& layout_item_
     }
     else
     {
-      const bool key_is_auto_increment = primary_key_field.get_auto_increment();
+      const bool key_is_auto_increment = primary_key_field->get_auto_increment();
 
       //If we would have to set an otherwise auto-increment key to add the record.
       if( key_is_auto_increment && !GlomConversions::value_is_empty(primary_key_value) )
@@ -982,32 +990,30 @@ bool Box_Data::add_related_record_for_field(const LayoutItem_Field& layout_item_
         //Create the related record:
         if(key_is_auto_increment)
         {
-          primary_key_value = generate_next_auto_increment(relationship.get_to_table(), primary_key_field.get_name());
+          primary_key_value = generate_next_auto_increment(relationship->get_to_table(), primary_key_field->get_name());
 
           //Generate the new key value;
         }
 
-        const Glib::ustring strQuery = "INSERT INTO " + relationship.get_to_table() + " (" + primary_key_field.get_name() + ") VALUES (" + primary_key_field.sql(primary_key_value) + ")";
+        const Glib::ustring strQuery = "INSERT INTO \"" + relationship->get_to_table() + "\" (\"" + primary_key_field->get_name() + "\") VALUES (" + primary_key_field->sql(primary_key_value) + ")";
         bool test = Query_execute(strQuery);
         if(test)
         {
           if(key_is_auto_increment)
           {
             //Set the key in the parent table
-            LayoutItem_Field item_from_key;
-            item_from_key.set_name(relationship.get_from_field());
+            sharedptr<LayoutItem_Field> item_from_key = sharedptr<LayoutItem_Field>::create();
+            item_from_key->set_name(relationship->get_from_field());
 
             //Show the new from key in the parent table's layout:
             set_entered_field_data(item_from_key, primary_key_value);
 
             //Set it in the database too:
-            Field field_from_key;
-            bool test = get_fields_for_table_one_field(relationship.get_from_table(), relationship.get_from_field(), field_from_key); //TODO_Performance.
-            if(test)
+            sharedptr<Field> field_from_key = get_fields_for_table_one_field(relationship->get_from_table(), relationship->get_from_field()); //TODO_Performance.
+            if(field_from_key)
             {
-              Field parent_primary_key_field;
-              bool test = get_field_primary_key(parent_primary_key_field);
-              if(!test)
+              sharedptr<Field> parent_primary_key_field = get_field_primary_key();
+              if(!parent_primary_key_field)
               {
                 g_warning("Box_Data::add_related_record_for_field(): get_field_primary_key() failed. table = %s", get_table_name().c_str());
                 return false;
@@ -1022,9 +1028,9 @@ bool Box_Data::add_related_record_for_field(const LayoutItem_Field& layout_item_
                 }
                 else
                 {
-                  const Glib::ustring strQuery = "UPDATE  " + relationship.get_from_table() + " SET " + relationship.get_from_field() + " = " + primary_key_field.sql(primary_key_value) +
-                    " WHERE " + relationship.get_from_table() + "." + parent_primary_key_field.get_name() + " = " + parent_primary_key_field.sql(parent_primary_key_value);
-                  bool test = Query_execute(strQuery);
+                  const Glib::ustring strQuery = "UPDATE \"" + relationship->get_from_table() + "\" SET \"" + relationship->get_from_field() + "\" = " + primary_key_field->sql(primary_key_value) +
+                    " WHERE \"" + relationship->get_from_table() + "\".\"" + parent_primary_key_field->get_name() + "\" = " + parent_primary_key_field->sql(parent_primary_key_value);
+                  const bool test = Query_execute(strQuery);
                   return test;
                 }
               }
