@@ -30,8 +30,8 @@ public:
   Dialog_Fields(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
   virtual ~Dialog_Fields();
 
-  virtual void init_db_details(const Glib::ustring& strDatabaseName, const Glib::ustring& strTableName);
-    
+  virtual bool init_db_details(const Glib::ustring& strTableName);
+
 protected:
 
   Box_DB_Table_Definition* m_box;
