@@ -37,6 +37,8 @@ main(int argc, char* argv[])
   button1.set_text("debug");
 
   FlowTable inner_table;
+  inner_table.set_padding(20);
+  inner_table.set_design_mode();
   inner_table.set_columns_count(1);
   inner_table.show();
   flowtable.add(inner_table);
@@ -68,7 +70,7 @@ main(int argc, char* argv[])
   Gtk::Entry button8; button8.set_text("eight");
   flowtable.add(button7, button8);
   button8.show();
-  button8.set_size_request(100, 100);
+  button8.set_size_request(100, 300);
 
   Gtk::Entry button9; button9.set_text("nine");;
   button9.show();
@@ -79,10 +81,11 @@ main(int argc, char* argv[])
   button10.show();
 
   window.add(flowtable);
+  flowtable.set_design_mode();
   flowtable.show();
 
 
-  flowtable.set_padding(6);
+  flowtable.set_padding(20);
 
   Gtk::Main::run(window);
 
